div[class^="highlight"] a {
    background-color: #E6E6E6;
}

div[class^="highlight"] a:hover {
    background-color: #ABECFC;
}

.wy-nav-content {
    max-width: 100%
}

/* For the link to edit the page on Github */
.source-link {
    float: right;
}

/* Don't let the edit link disappear on mobile. */
@media screen and (max-width: 480px) {
    .wy-breadcrumbs li.source-link {
        float:none;
        display: block;
        margin-top: 20px;
    }
}
